/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message;

import biz.massivedynamics.modger.Logger;
import biz.massivedynamics.modger.formatter.Formatter;

/**
 * A {@link Message} class that encapsulates a {@link String} and {@link MessageType}
 * 
 * @author Cruz Bishop
 * @since 1.0.0.0
 */
public class Message implements Cloneable {
    
    /**
     * Creates a new {@link Message} with the regular {@link MessageType}
     * 
     * @param logger The {@link Logger} to use
     * @param message The {@link String} message to use
     * @since 1.0.0.0
     */
    public Message(Logger logger, String message) {
        this(logger, message, MessageType.REGULAR);
    }
    
    /**
     * Creates a new {@link Message}
     * 
     * @param logger The {@link Logger} to use
     * @param message The {@link String} message to use
     * @param type The {@link MessageType} to use
     * @since 1.0.0.0
     */
    public Message(Logger logger, String message, MessageType type) {
        this.logger = logger;
        this.rawMessage = message;
        this.type = type;
    }
    
    /**
     * The {@link Logger} being used
     * 
     * @since 1.0.0.0
     */
    private Logger logger;
    
    /**
     * Gets the {@link Logger} being used
     * 
     * @return The {@link Logger}
     * @since 1.0.0.0
     */
    public Logger getLogger() {
        return this.logger;
    }
    
    /**
     * Gets the raw {@link String} message
     * 
     * @return The message
     * @since 1.0.0.0
     * @deprecated Please use the raw message instead!
     */
    @Deprecated
    public String getMessage() {
        return this.rawMessage;
    }
    
    /**
     * Sets the {@link String} message
     * 
     * @param message The message to use
     * @since 1.0.0.0
     * @deprecated Please use the raw message instead!
     */
    @Deprecated
    public void setMessage(String message) {
        this.rawMessage = message;
    }
    
    /**
     * The raw {@link String} message
     * 
     * @since 1.1.0.0
     */
    private String rawMessage;
    
    /**
     * Gets the raw {@link String} message
     * 
     * @return The raw {@link String} message
     * @since 1.1.0.0
     */
    public String getRawMessage() {
        return this.rawMessage;
    }
    
    /**
     * Sets the raw {@link String} message and reformats the formatted one
     * 
     * @param message The raw {@link String} message to set
     * @since 1.1.0.0
     */
    public void setRawMessage(String message) {
        this.rawMessage = message;
        
        //Also reformat the formatted message
        this.reformatMessage();
    }
    
    /**
     * The formatted {@link String} message
     * 
     * @since 1.1.0.0
     */
    private String formattedMessage;
    
    /**
     * Reformats the formatted {@link String} message with the {@link Logger}'s {@link Message} {@link Formatter}
     * <br /> <br />
     * Please note that you don't need to do this manually, as it is done
     * automatically when setting the raw {@link String} message
     * 
     * @since 1.1.0.0
     */
    public void reformatMessage() {
        this.formattedMessage = this.logger.getMessageFormatter().format(this).getFormattedMessage();
    }
    
    /**
     * Sets the formatted {@link String} message
     * 
     * @param message The formatted {@link String} message
     * @since 1.1.0.0
     */
    public void setFormattedMessage(String message) {
        this.formattedMessage = message;
    }
    
    /**
     * Gets the dynamically formatted {@link String} message
     * 
     * @return The formatted {@link String} message
     * @since 1.1.0.0
     */
    public String getFormattedMessage() {
        return this.formattedMessage;
    }
    
    /**
     * The {@link MessageType} of this {@link Message}
     * 
     * @since 1.0.0.0
     */
    private MessageType type;
    
    /**
     * Gets this {@link Message}'s {@link MessageType}
     * 
     * @return The {@link MessageType}
     * @since 1.0.0.0
     */
    public MessageType getType() {
        return this.type;
    }
    
    /**
     * Sets the {@link MessageType}
     * 
     * @param type The {@link MessageType} to use
     * @since 1.0.0.0
     */
    public void setType(MessageType type) {
        this.type = type;
    }
    
    /**
     * Clones this {@link Message}
     * 
     * @return The cloned {@link Message}
     * @since 1.0.0.0
     */
    @Override
    public Message clone() {
        return new Message(this.getLogger(), this.getRawMessage(), this.getType());
    }
    
    /**
     * Gets the length of the raw {@link String} message
     * 
     * @return The length
     * @since 1.0.0.0
     */
    public int getLength() {
        return this.getRawMessage().length();
    }

    
}
