/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message.filter.impl;

import biz.massivedynamics.modger.message.Message;
import biz.massivedynamics.modger.message.filter.MessageFilter;

/**
 * Filters {@link Message}s according to their length.
 *
 * @author Cruz Bishop
 * @since 1.0.0.0
 */
public class MessageLengthFilter extends MessageFilter {
    
    /**
     * Creates a new {@link MessageLengthFilter}
     * 
     * @param minimum The minimum length to allow
     * @param maximum The maximum length to allow
     * @since 1.0.0.0
     */
    public MessageLengthFilter(int minimum, int maximum) {
        //Set the minimum length
        this.minimumLength = minimum;
        //Set the maximum length
        this.maximumLength = maximum;
    }
    
    /**
     * The minimum length a {@link Message} is allowed to be
     * 
     * @since 1.0.0.0
     */
    private int minimumLength;
    
    /**
     * Gets the minimum length that a {@link Message} is allowed to be
     * 
     * @return The minimum length
     * @since 1.0.0.0
     */
    public int getMinimumLength() {
        return this.minimumLength;
    }
    
    /**
     * Sets the minimum length that a {@link Message} is allowed to be
     * 
     * @param minimum The minimum length
     * @since 1.0.0.0
     */
    public void setMinimumLength(int minimum) {
        this.minimumLength = minimum;
    }
    
    /**
     * The maximum length that a {@link Message} is allowed to be
     * 
     * @since 1.0.0.0
     */
    private int maximumLength;
    
    /**
     * Gets the maximum length that a {@link Message} is allowed to be
     * 
     * @return The maximum length
     * @since 1.0.0.0
     */
    public int getMaximumLength() {
        return this.maximumLength;
    }
    
    /**
     * Sets the maximum length that a {@link Message} is allowed to be
     * 
     * @param maximum The maximum length
     * @since 1.0.0.0
     */
    public void setMaximumLength(int maximum) {
        this.maximumLength = maximum;
    }

    /**
     * Checks to see if the provided {@link Message} is acceptable for use
     * 
     * @param message The {@link Message} to check
     * @return True if the {@link Message} meets all requirements, otherwise false
     */
    @Override
    public boolean isAcceptable(Message message) {
        if ((message.getLength() < this.minimumLength) ||
                (message.getLength() > this.maximumLength)) {
            return false;
        }
        
        return true;
    }
    
}
