/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message.handler.impl;

import biz.massivedynamics.modger.Modger;
import biz.massivedynamics.modger.message.Message;
import biz.massivedynamics.modger.message.handler.MessageHandler;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SeekableByteChannel;
import java.nio.charset.Charset;
import java.nio.file.*;

/**
 * A {@link MessageHandler} that handles {@link Message}s and attached 
 * {@link Throwable}s.
 * <p>
 *     The data is passed to a {@link SeekableByteChannel}, which is the NIO
 *     equivalent of a {@link java.io.File}.
 *     <br />
 *     This {@link SeekableByteChannel} points to a UTF-8 text file.
 * </p>
 *
 * @author Cruz Bishop
 * @since 1.0.0.0
 */
public class TextFileMessageHandler extends MessageHandler {
    
    /**
     * Creates a new {@link TextFileMessageHandler} at the specified {@link Path}
     * 
     * @param path The {@link Path} to use
     * @throws IOException Any {@link IOException}s that occur
     * @since 1.0.0.0
     */
    public TextFileMessageHandler(Path path) throws IOException {
        //First, check to see if the path exists
        if (Files.exists(path)) {
            //See if it's a file, not a directory
            if (Files.isDirectory(path)) {
                //Well, damn. We can't write to a directory!
                throw new NoSuchFileException(path.toString());
            }
        } else {
            //Create the directories
            Files.createDirectories(path.getParent());
            //Create the file
            Files.createFile(path);
        } 
        
        //Check to see if the file is writable
        if (Files.isWritable(path)) {
            //We can keep going! :)
            this.path = path;
            this.channel = Files.newByteChannel(path, StandardOpenOption.CREATE, StandardOpenOption.APPEND, StandardOpenOption.WRITE);
        } else throw new AccessDeniedException(path.toString());
    }
    
    /**
     * The {@link Path} used to locate the text file
     * 
     * @since 1.0.0.0
     */
    private Path path = null;
    
    /**
     * Gets the {@link Path} used to locate the text file
     * 
     * @return The {@link Path}
     * @since 1.0.0.0
     */
    public Path getPath() {
        return this.path;
    }
    
    /**
     * The {@link SeekableByteChannel} that is used for the text file
     * 
     * @since 1.0.0.0
     */
    private final SeekableByteChannel channel;
    
    /**
     * Gets the {@link SeekableByteChannel} used for the text file
     * 
     * @return The {@link SeekableByteChannel}
     * @since 1.0.0.0
     */
    public SeekableByteChannel getChannel() {
        return this.channel;
    }

    /**
     * Submits a {@link Message} to be inserted into the {@link SeekableByteChannel}
     * 
     * @param message The {@link Message} to submit
     * @throws Exception Any {@link Exception}s that are thrown
     * @since 1.0.0.0
     */
    @Override
    public void submit(Message message) throws Exception {
        synchronized(this.channel) {
            this.channel.write(ByteBuffer.wrap((message.getFormattedMessage() 
                    + Modger.getLineSeparator()).getBytes("UTF-8")));
        }
    }

    /**
     * Submits a {@link Message} with an attached {@link Throwable} cause to the {@link SeekableByteChannel}
     * 
     * @param message The {@link Message}
     * @param cause The attached {@link Throwable} cause
     * @throws Exception Any {@link Exception}s that occur
     * @sicne 1.1.0.0
     */
    @Override
    public void submit(Message message, Throwable cause) throws Exception {
        synchronized(this.channel) {
            this.channel.write(ByteBuffer.wrap((message.getFormattedMessage() + 
                    Modger.getLineSeparator() + 
                    message.getLogger().getThrowableFormatter().format(cause))
                .getBytes(Charset.forName("UTF-8"))));
        }
    }
    
}
